@@ -176,22 +176,29 @@ function substitute_name(O, st)
     end
 end
 
+function _is_tuple_or_array_of_symbolics(O)
+    return O isa CodegenPrimitive ||
+        (symbolic_type(O) != NotSymbolic() && !(O isa Union{Symbol, Expr})) ||
+        _is_array_of_symbolics(O) ||
+        _is_tuple_of_symbolics(O)
+end
+
 function _is_array_of_symbolics(O)
     # O is an array, not a symbolic array, and either has a non-symbolic eltype or contains elements that are
     # symbolic or arrays of symbolics
     return O isa AbstractArray && symbolic_type(O) == NotSymbolic() &&
-        (symbolic_type(eltype(O)) != NotSymbolic() ||
-        any(x -> x isa CodegenPrimitive || symbolic_type(x) != NotSymbolic() || _is_array_of_symbolics(x), O))
+        (symbolic_type(eltype(O)) != NotSymbolic() && !(eltype(O) <: Union{Symbol, Expr}) ||
+        any(_is_tuple_or_array_of_symbolics, O))
 end
 
 # workaround for https://github.com/JuliaSparse/SparseArrays.jl/issues/599
 function _is_array_of_symbolics(O::SparseMatrixCSC)
-    return symbolic_type(eltype(O)) != NotSymbolic() ||
-        any(x -> x isa CodegenPrimitive || symbolic_type(x) != NotSymbolic() || _is_array_of_symbolics(x), findnz(O)[3])
+    return symbolic_type(eltype(O)) != NotSymbolic() && !(eltype(O) <: Union{Symbol, Expr}) ||
+        any(_is_tuple_or_array_of_symbolics, findnz(O)[3])
 end
 
 function _is_tuple_of_symbolics(O::Tuple)
-    return any(x -> x isa CodegenPrimitive || symbolic_type(x) != NotSymbolic() || _is_array_of_symbolics(x) || _is_tuple_of_symbolics(x), O)
+    return any(_is_tuple_or_array_of_symbolics, O)
 end
 _is_tuple_of_symbolics(O) = false
 
@@ -595,20 +602,20 @@ end
 
 ## SArray
 @inline function create_array(::Type{<:SArray}, ::Nothing, nd::Val, ::Val{dims}, elems...) where dims
-    SArray{Tuple{dims...}}(elems...)
+    SArray{Tuple{dims...}}(elems)
 end
 
 @inline function create_array(::Type{<:SArray}, T, nd::Val, ::Val{dims}, elems...) where dims
-    SArray{Tuple{dims...}, T}(elems...)
+    SArray{Tuple{dims...}, T}(elems)
 end
 
 ## MArray
 @inline function create_array(::Type{<:MArray}, ::Nothing, nd::Val, ::Val{dims}, elems...) where dims
-    MArray{Tuple{dims...}}(elems...)
+    MArray{Tuple{dims...}}(elems)
 end
 
 @inline function create_array(::Type{<:MArray}, T, nd::Val, ::Val{dims}, elems...) where dims
-    MArray{Tuple{dims...}, T}(elems...)
+    MArray{Tuple{dims...}, T}(elems)
 end
 
 ## We use a separate type for Sparse Arrays to sidestep the need for
@@ -659,6 +666,21 @@ function toexpr(a::MakeTuple, st)
     :(($(toexpr.(a.elems, (st,))...),))
 end
 
+"""
+    Multithreaded
+
+A parallelism type for `SpawnFetch` that uses Julia's threading system.
+
+When used with `SpawnFetch{Multithreaded}`, expressions are executed
+in parallel using `Threads.@spawn`.
+
+# Examples
+```julia
+julia> SpawnFetch{Multithreaded}([func1, func2], combine_func)
+```
+
+See also: [`SpawnFetch`](@ref)
+"""
 struct Multithreaded end
 """
     SpawnFetch{ParallelType}(funcs [, args], reduce)
@@ -737,7 +759,16 @@ end
 
 ### Common subexprssion evaluation
 
-@inline newsym(::Type{T}) where T = Sym{T}(gensym("cse"))
+"""
+    newsym!(state::CSEState, ::Type{T})
+
+Generates new symbol of type `T` with unique name in `state`.
+"""
+@inline function newsym!(state, ::Type{T}) where T 
+    name = "##cse#$(state.varid[])"
+    state.varid[] += 1
+    Sym{T}(Symbol(name))
+end
 
 """
     $(TYPEDSIGNATURES)
@@ -769,11 +800,16 @@ struct CSEState
     A mapping of symbolic expression to the LHS in `sorted_exprs` that computes it.
     """
     visited::IdDict{Any, Any}
+    """
+    Integer counter, used to generate unique names for intermediate variables.
+    """
+    varid::Ref{Int}
 end
 
-CSEState() = CSEState(Union{Assignment, DestructuredArgs}[], IdDict())
+CSEState() = CSEState(Union{Assignment, DestructuredArgs}[], IdDict(), Ref(1))
 
-Base.copy(x::CSEState) = CSEState(copy(x.sorted_exprs), copy(x.visited))
+# the copy still references the same `varid` Ref to work in nested scopes
+Base.copy(x::CSEState) = CSEState(copy(x.sorted_exprs), copy(x.visited), x.varid)
 
 """
     $(TYPEDSIGNATURES)
@@ -813,8 +849,22 @@ end
 """
     $(TYPEDSIGNATURES)
 
-Perform Common Subexpression Elimination on the given expression `expr`. Return an
-equivalent `expr` with optimized computation.
+Perform common subexpression elimination on an expression.
+
+This optimization identifies repeated subexpressions and replaces them with
+variables to avoid redundant computation.
+
+# Arguments
+- `expr`: The expression to optimize
+
+# Returns
+An optimized expression with common subexpressions eliminated
+
+# Examples
+```julia
+julia> expr = :(sin(x) + sin(x) * cos(y))
+julia> cse(expr)  # sin(x) is computed only once
+```
 """
 function cse(expr)
     state = CSEState()
@@ -861,13 +911,13 @@ function cse!(expr::Symbolic, state::CSEState)
                 (_is_array_of_symbolics(arg) || _is_tuple_of_symbolics(arg))
                 if arg isa Tuple
                     new_arg = cse!(MakeTuple(arg), state)
-                    sym = newsym(Tuple{symtype.(arg)...})
+                    sym = newsym!(state, Tuple{symtype.(arg)...})
                 elseif issparse(arg)
                     new_arg = cse!(MakeSparseArray(arg), state)
-                    sym = newsym(AbstractSparseArray{symtype(eltype(arg)), indextype(arg), ndims(arg)})
+                    sym = newsym!(state, AbstractSparseArray{symtype(eltype(arg)), indextype(arg), ndims(arg)})
                 else
                     new_arg = cse!(MakeArray(arg, typeof(arg)), state)
-                    sym = newsym(AbstractArray{symtype(eltype(arg)), ndims(arg)})
+                    sym = newsym!(state, AbstractArray{symtype(eltype(arg)), ndims(arg)})
                 end
                 push!(state.sorted_exprs, sym ← new_arg)
                 state.visited[arg] = sym
@@ -878,7 +928,7 @@ function cse!(expr::Symbolic, state::CSEState)
         # use `term` instead of `maketerm` because we only care about the operation being performed
         # and not the representation. This avoids issues with `newsym` symbols not having sizes, etc.
         new_expr = term(operation(expr), args...; type = symtype(expr))
-        sym = newsym(symtype(new_expr))
+        sym = newsym!(state, symtype(new_expr))
         push!(state.sorted_exprs, sym ← new_expr)
         return sym
     end
