@@ -22,10 +22,13 @@ const EMPTY_HASH = RefValue(UInt(0))
 const EMPTY_DICT = sdict()
 const EMPTY_DICT_T = typeof(EMPTY_DICT)
 const ENABLE_HASHCONSING = Ref(true)
+const TID = Union{IDType, Nothing}
+const DID = nothing
 
 @compactify show_methods=false begin
     @abstract mutable struct BasicSymbolic{T} <: Symbolic{T}
         metadata::Metadata     = NO_METADATA
+        id::RefValue{TID} = Ref{TID}(DID)
     end
     mutable struct Sym{T} <: BasicSymbolic{T}
         name::Symbol           = :OOF
@@ -87,11 +90,9 @@ function exprtype(x::BasicSymbolic)
     end
 end
 
-const wvd = TaskLocalValue{WeakValueDict{UInt, BasicSymbolic}}(WeakValueDict{UInt, BasicSymbolic})
-
 # Same but different error messages
 @noinline error_on_type() = error("Internal error: unreachable reached!")
-@noinline error_sym() = error("Sym doesn't have a operation or arguments!")
+@noinline error_sym() = error("Sym doesn't have an operation or any arguments!")
 @noinline error_property(E, s) = error("$E doesn't have field $s")
 
 # We can think about bits later
@@ -107,11 +108,11 @@ function ConstructionBase.setproperties(obj::BasicSymbolic{T}, patch::NamedTuple
     # Call outer constructor because hash consing cannot be applied in inner constructor
     @compactified obj::BasicSymbolic begin
         Sym => Sym{T}(nt_new.name; nt_new...)
-        Term => Term{T}(nt_new.f, nt_new.arguments; nt_new..., hash = RefValue(UInt(0)), hash2 = RefValue(UInt(0)))
-        Add => Add(T, nt_new.coeff, nt_new.dict; nt_new..., hash = RefValue(UInt(0)), hash2 = RefValue(UInt(0)))
-        Mul => Mul(T, nt_new.coeff, nt_new.dict; nt_new..., hash = RefValue(UInt(0)), hash2 = RefValue(UInt(0)))
-        Div => Div{T}(nt_new.num, nt_new.den, nt_new.simplified; nt_new..., hash = RefValue(UInt(0)), hash2 = RefValue(UInt(0)))
-        Pow => Pow{T}(nt_new.base, nt_new.exp; nt_new..., hash = RefValue(UInt(0)), hash2 = RefValue(UInt(0)))
+        Term => Term{T}(nt_new.f, nt_new.arguments; nt_new..., hash = RefValue(UInt(0)), hash2 = RefValue(UInt(0)), id = Ref{TID}(DID))
+        Add => Add(T, nt_new.coeff, nt_new.dict; nt_new..., hash = RefValue(UInt(0)), hash2 = RefValue(UInt(0)), id = Ref{TID}(DID))
+        Mul => Mul(T, nt_new.coeff, nt_new.dict; nt_new..., hash = RefValue(UInt(0)), hash2 = RefValue(UInt(0)), id = Ref{TID}(DID))
+        Div => Div{T}(nt_new.num, nt_new.den, nt_new.simplified; nt_new..., hash = RefValue(UInt(0)), hash2 = RefValue(UInt(0)), id = Ref{TID}(DID))
+        Pow => Pow{T}(nt_new.base, nt_new.exp; nt_new..., hash = RefValue(UInt(0)), hash2 = RefValue(UInt(0)), id = Ref{TID}(DID))
         _ => Unityper.rt_constructor(obj){T}(;nt_new...)
     end
 end
@@ -148,6 +149,37 @@ end
 
 @inline head(x::BasicSymbolic) = operation(x)
 
+"""
+    sorted_arguments(x::BasicSymbolic)
+
+Get the arguments of a symbolic expression in canonical sorted order.
+
+For commutative operations like addition and multiplication, the arguments
+are sorted according to a canonical ordering. This ensures that equivalent
+expressions have the same representation.
+
+# Arguments
+- `x::BasicSymbolic`: The symbolic expression
+
+# Returns
+A vector of the arguments in sorted order. For non-commutative operations,
+returns the arguments in their original order.
+
+# Examples
+```julia
+julia> @syms x y z
+(x, y, z)
+
+julia> expr = x + z + y
+x + y + z
+
+julia> sorted_arguments(expr)
+3-element Vector{Any}:
+ x
+ y
+ z
+```
+"""
 @cache function TermInterface.sorted_arguments(x::BasicSymbolic)::Vector{Any}
     args = copy(arguments(x))
     @compactified x::BasicSymbolic begin
@@ -255,6 +287,7 @@ end
 
 function Base.isequal(a::BasicSymbolic{T}, b::BasicSymbolic{S}) where {T,S}
     a === b && return true
+    a.id == b.id && a.id != 0 && return true
 
     E = exprtype(a)
     E === exprtype(b) || return false
@@ -298,6 +331,7 @@ function.
 """
 function isequal_with_metadata(a::BasicSymbolic{T}, b::BasicSymbolic{S})::Bool where {T, S}
     a === b && return true
+    a.id == b.id && a.id != 0 && return true
 
     E = exprtype(a)
     E === exprtype(b) || return false
@@ -513,6 +547,12 @@ end
 ### Constructors
 ###
 
+const wvd = TaskLocalValue{WeakCacheSet{BasicSymbolic}}(WeakCacheSet{BasicSymbolic})
+
+function generate_id()
+    return IDType()
+end
+
 """
 $(TYPEDSIGNATURES)
 
@@ -520,7 +560,7 @@ Implements hash consing (flyweight design pattern) for `BasicSymbolic` objects.
 
 This function checks if an equivalent `BasicSymbolic` object already exists. It uses a 
 custom hash function (`hash2`) incorporating metadata and symtypes to search for existing 
-objects in a `WeakValueDict` (`wvd`). Due to the possibility of hash collisions (where 
+objects in a `WeakCacheSet` (`wcs`). Due to the possibility of hash collisions (where 
 different objects produce the same hash), a custom equality check (`isequal_with_metadata`) 
 which includes metadata comparison, is used to confirm the equivalence of objects with 
 matching hashes. If an equivalent object is found, the existing object is returned; 
@@ -528,7 +568,7 @@ otherwise, the input `s` is returned. This reduces memory usage, improves compil
 for runtime code generation, and supports built-in common subexpression elimination, 
 particularly when working with symbolic objects with metadata.
 
-Using a `WeakValueDict` ensures that only weak references to `BasicSymbolic` objects are 
+Using a `WeakCacheSet` ensures that only weak references to `BasicSymbolic` objects are 
 stored, allowing objects that are no longer strongly referenced to be garbage collected. 
 Custom functions `hash2` and `isequal_with_metadata` are used instead of `Base.hash` and 
 `Base.isequal` to accommodate metadata without disrupting existing tests reliant on the 
@@ -538,25 +578,24 @@ function BasicSymbolic(s::BasicSymbolic)::BasicSymbolic
     if !ENABLE_HASHCONSING[]
         return s
     end
+
     cache = wvd[]
-    h = hash2(s)
-    k = get!(cache, h, s)
-    if isequal_with_metadata(k, s)
-        return k
-    else
-        return s
+    k = getkey!(cache, s)
+    if k.id[] === nothing
+        k.id[] = generate_id()
     end
+    return k
 end
 
 function Sym{T}(name::Symbol; kw...) where {T}
-    s = Sym{T}(; name, kw...)
+    s = Sym{T}(; name, kw..., id = Ref{TID}(DID))
     BasicSymbolic(s)
 end
 
 function Term{T}(f, args; kw...) where T
     args = SmallV{Any}(args)
 
-    s = Term{T}(;f=f, arguments=args, hash=Ref(UInt(0)), hash2=Ref(UInt(0)), kw...)
+    s = Term{T}(;f=f, arguments=args, hash=Ref(UInt(0)), hash2=Ref(UInt(0)), kw..., id = Ref{TID}(DID))
     BasicSymbolic(s)
 end
 
@@ -586,7 +625,7 @@ function Add(::Type{T}, coeff, dict; metadata=NO_METADATA, kw...) where T
         end
     end
 
-    s = Add{T}(; coeff, dict, hash=Ref(UInt(0)), hash2=Ref(UInt(0)), metadata, arguments=SmallV{Any}(), kw...)
+    s = Add{T}(; coeff, dict, hash=Ref(UInt(0)), hash2=Ref(UInt(0)), metadata, arguments=SmallV{Any}(), kw..., id = Ref{TID}(DID))
     BasicSymbolic(s)
 end
 
@@ -604,7 +643,7 @@ function Mul(T, a, b; metadata=NO_METADATA, kw...)
     else
         coeff = a
         dict = b
-        s = Mul{T}(; coeff, dict, hash=Ref(UInt(0)), hash2=Ref(UInt(0)), metadata, arguments=SmallV{Any}(), kw...)
+        s = Mul{T}(; coeff, dict, hash=Ref(UInt(0)), hash2=Ref(UInt(0)), metadata, arguments=SmallV{Any}(), kw..., id = Ref{TID}(DID))
         BasicSymbolic(s)
     end
 end
@@ -672,7 +711,7 @@ function Div{T}(n, d, simplified=false; metadata=nothing, kwargs...) where {T}
         end
     end
 
-    s = Div{T}(; num=n, den=d, simplified, arguments=SmallV{Any}(), metadata)
+    s = Div{T}(; num=n, den=d, simplified, arguments=SmallV{Any}(), metadata, id = Ref{TID}(DID))
     BasicSymbolic(s)
 end
 
@@ -692,7 +731,7 @@ function Pow{T}(a, b; metadata=NO_METADATA, kwargs...) where {T}
     b = unwrap(b)
     _iszero(b) && return 1
     _isone(b) && return a
-    s = Pow{T}(; base=a, exp=b, arguments=SmallV{Any}(), metadata)
+    s = Pow{T}(; base=a, exp=b, arguments=SmallV{Any}(), metadata, id = Ref{TID}(DID))
     BasicSymbolic(s)
 end
 
@@ -787,6 +826,31 @@ function makepow(a, b)
     return (base, exp)
 end
 
+"""
+    term(f, args...; type = nothing)
+
+Create a symbolic term with operation `f` and arguments `args`.
+
+# Arguments
+- `f`: The operation or function head of the term
+- `args...`: The arguments to the operation
+- `type`: Optional type specification for the term. If not provided, the type is inferred using `promote_symtype`.
+
+# Examples
+```julia
+julia> @syms x y
+(x, y)
+
+julia> term(+, x, y)
+x + y
+
+julia> term(sin, x)
+sin(x)
+
+julia> term(^, x, 2)
+x^2
+```
+"""
 function term(f, args...; type = nothing)
     args = SmallV{Any}(args)
     if type === nothing
@@ -847,13 +911,13 @@ function basicsymbolic(f, args, stype, metadata)
     elseif all(x->symtype(x) <: Number, args)
         if f === (+)
             res = +(args...)
-            if isadd(res) || isterm(res)
+            if isadd(res) || (isterm(res) && operation(res) == (+))
                 @set! res.metadata = metadata
             end
             res
         elseif f == (*)
             res = *(args...)
-            if ismul(res) || isterm(res)
+            if ismul(res) || (isterm(res) && operation(res) == (*))
                 @set! res.metadata = metadata
             end
             res
@@ -885,6 +949,27 @@ end
 metadata(s::Symbolic) = s.metadata
 metadata(s::Symbolic, meta) = Setfield.@set! s.metadata = meta
 
+"""
+    hasmetadata(s::Symbolic, ctx)
+
+Check if a symbolic expression has metadata for a given context.
+
+# Arguments
+- `s::Symbolic`: The symbolic expression to check
+- `ctx`: The metadata context key (typically a DataType)
+
+# Returns
+- `true` if the expression has metadata for the given context, `false` otherwise
+
+# Examples
+```julia
+julia> @syms x
+x
+
+julia> hasmetadata(x, Float64)
+false
+```
+"""
 function hasmetadata(s::Symbolic, ctx)
     metadata(s) isa AbstractDict && haskey(metadata(s), ctx)
 end
@@ -903,6 +988,30 @@ _issafecanon(::typeof(^), s) = !iscall(s) || !(operation(s) in (*, ^))
 
 issafecanon(f, ss...) = all(x->issafecanon(f, x), ss)
 
+"""
+    getmetadata(s::Symbolic, ctx)
+
+Retrieve metadata associated with a symbolic expression for a given context.
+
+# Arguments
+- `s::Symbolic`: The symbolic expression
+- `ctx`: The metadata context key (typically a DataType)
+
+# Returns
+The metadata value associated with the given context
+
+# Throws
+- `ArgumentError` if the expression does not have metadata for the given context
+
+# Examples
+```julia
+julia> @syms x::Float64
+x
+
+julia> getmetadata(x, symtype)  # Get the type metadata
+Float64
+```
+"""
 function getmetadata(s::Symbolic, ctx)
     md = metadata(s)
     if md isa AbstractDict
@@ -912,6 +1021,29 @@ function getmetadata(s::Symbolic, ctx)
     end
 end
 
+"""
+    getmetadata(s::Symbolic, ctx, default)
+
+Retrieve metadata associated with a symbolic expression for a given context,
+returning a default value if not found.
+
+# Arguments
+- `s::Symbolic`: The symbolic expression
+- `ctx`: The metadata context key (typically a DataType)
+- `default`: The default value to return if metadata is not found
+
+# Returns
+The metadata value associated with the given context, or `default` if not found
+
+# Examples
+```julia
+julia> @syms x
+x
+
+julia> getmetadata(x, Float64, "no type")
+"no type"
+```
+"""
 function getmetadata(s::Symbolic, ctx, default)
     md = metadata(s)
     md isa AbstractDict ? get(md, ctx, default) : default
@@ -940,6 +1072,31 @@ function assocmeta(d::Base.ImmutableDict, ctx, val)::ImmutableDict{DataType,Any}
     Base.ImmutableDict{DataType, Any}(d, ctx, val)
 end
 
+"""
+    setmetadata(s::Symbolic, ctx::DataType, val)
+
+Set metadata for a symbolic expression in a given context.
+
+# Arguments
+- `s::Symbolic`: The symbolic expression
+- `ctx::DataType`: The metadata context key
+- `val`: The metadata value to set
+
+# Returns
+A new symbolic expression with the updated metadata
+
+# Examples
+```julia
+julia> @syms x
+x
+
+julia> x_with_meta = setmetadata(x, Float64, "custom value")
+x
+
+julia> getmetadata(x_with_meta, Float64)
+"custom value"
+```
+"""
 function setmetadata(s::Symbolic, ctx::DataType, val)
     if s.metadata isa AbstractDict
         @set s.metadata = assocmeta(s.metadata, ctx, val)
@@ -1126,6 +1283,31 @@ function show_term(io::IO, t)
     return nothing
 end
 
+"""
+    showraw([io::IO], t)
+
+Display the raw structure of a symbolic expression without simplification.
+
+This function shows the internal structure of symbolic expressions without applying
+any simplification rules, which is useful for debugging and understanding the
+exact form of an expression.
+
+# Arguments
+- `io::IO`: Optional IO stream to write to (defaults to stdout)
+- `t`: The symbolic expression to display
+
+# Examples
+```julia
+julia> @syms x
+x
+
+julia> expr = x + x + x
+3x
+
+julia> showraw(expr)  # Shows the unsimplified structure
+x + x + x
+```
+"""
 showraw(io, t) = Base.show(IOContext(io, :simplify=>false), t)
 showraw(t) = showraw(stdout, t)
 
@@ -1346,39 +1528,79 @@ sub_t(a,b) = promote_symtype(-, symtype(a), symtype(b))
 sub_t(a) = promote_symtype(-, symtype(a))
 
 import Base: (+), (-), (*), (//), (/), (\), (^)
-function +(a::SN, b::SN)
-    !issafecanon(+, a,b) && return term(+, a, b) # Don't flatten if args have metadata
-    if isadd(a) && isadd(b)
-        return Add(add_t(a,b),
-                   a.coeff + b.coeff,
-                   _merge(+, a.dict, b.dict, filter=_iszero))
-    elseif isadd(a)
-        coeff, dict = makeadd(1, 0, b)
-        return Add(add_t(a,b), a.coeff + coeff, _merge(+, a.dict, dict, filter=_iszero))
-    elseif isadd(b)
-        return b + a
+
+function +(a::SN, bs::SN...)
+    isempty(bs) && return a
+    # entries where `!issafecanon`
+    unsafes = SmallV{Any}()
+    # coeff and dict of the `Add`
+    coeff = 0
+    dict = sdict()
+    # type of the `Add`
+    T = symtype(a)
+
+    # handle `a` separately
+    if issafecanon(+, a)
+        if isadd(a)
+            coeff = a.coeff
+            dict = copy(a.dict)
+        elseif ismul(a)
+            v = a.coeff
+            a′ = Mul(symtype(a), 1, copy(a.dict); metadata = a.metadata)
+            dict[a′] = v
+        else
+            dict[a] = 1
+        end
+    else
+        push!(unsafes, a)
     end
-    coeff, dict = makeadd(1, 0, a, b)
-    Add(add_t(a,b), coeff, dict)
+
+    for b in bs
+        T = promote_symtype(+, T, symtype(b))
+        if !issafecanon(+, b)
+            push!(unsafes, b)
+            continue
+        end
+        if isadd(b)
+            coeff += b.coeff
+            for (k, v) in b.dict
+                dict[k] = get(dict, k, 0) + v
+            end
+        elseif ismul(b)
+            v = b.coeff
+            b′ = Mul(symtype(b), 1, copy(b.dict); metadata = b.metadata)
+            dict[b′] = get(dict, b′, 0) + v
+        else
+            dict[b] = get(dict, b, 0) + 1
+        end
+    end
+    # remove entries multiplied by zero
+    filter!(dict) do kvp
+        !iszero(kvp[2])
+    end
+
+    result = isempty(dict) ? coeff : Add(T, coeff, dict)
+    if !isempty(unsafes)
+        push!(unsafes, result)
+        result = Term{T}(+, unsafes)
+    end
+    return result
 end
 
-function +(a::Number, b::SN)
-    tmp = unwrap(a)
-    if tmp !== a
-        return tmp + b
-    end
-    !issafecanon(+, b) && return term(+, a, b) # Don't flatten if args have metadata
+function +(a::Number, b::SN, bs::SN...)
+    b = +(b, bs...)
+    issafecanon(+, b) || return term(+, a, b)
     iszero(a) && return b
     if isadd(b)
-        Add(add_t(a,b), a + b.coeff, b.dict)
+        Add(add_t(a, b), a + b.coeff, b.dict)
     else
-        Add(add_t(a,b), makeadd(1, a, b)...)
+        Add(add_t(a, b), makeadd(1, a, b)...)
     end
 end
 
-+(a::SN, b::Number) = b + a
-
-+(a::SN) = a
+function +(a::SN, b::Number, bs::SN...)
+    return +(b, a, bs...)
+end
 
 function -(a::SN)
     !issafecanon(*, a) && return term(-, a)
@@ -1482,12 +1704,17 @@ function ^(a::SN, b)
     if b isa Number && iszero(b)
         # fast path
         1
-    elseif b isa Number && b < 0
+    elseif b isa Real && b < 0
         Div(1, a ^ (-b))
     elseif ismul(a) && b isa Number
-        coeff = unstable_pow(a.coeff, b)
-        Mul(promote_symtype(^, symtype(a), symtype(b)),
-            coeff, mapvalues((k, v) -> b*v, a.dict))
+        if a.coeff isa Real && a.coeff < 0 && !isinteger(b)
+            coeff = unstable_pow(-a.coeff, b)
+            coeff * Pow(Mul(promote_symtype(^, symtype(a), symtype(b)), -1, copy(a.dict)), b)
+        else
+            coeff = unstable_pow(a.coeff, b)
+            Mul(promote_symtype(^, symtype(a), symtype(b)),
+                coeff, mapvalues((k, v) -> b*v, a.dict))
+        end
     else
         Pow(a, b)
     end
